from hydroplane.hydro import hydro
from hydroplane.components.card import HydroCard
from dash import html, dcc
from dash.dependencies import Input, Output, State, MATCH, ALL 

class HydroSimple:
    def __init__(self):
        self.cards = HydroCard()

    def hydro_simple(self):
        card_body = html.Section(id="simple", children=[
            "SIMPLE"
        ])
        simple = self.cards.hydro_plain_card(1, "large", card_body)
        return simple

 


import os

from hydroplane import hydro, hydro_router, Hydroplane 
from hydroplane import Input, Output, State, MATCH, ALL

# INITIALIZE HYDROPLANE
hydroplane_postgres_env = {
    "host": os.environ['POSTGRES_HOST'],
    "port": os.environ['POSTGRES_PORT'],
    "database": os.environ['POSTGRES_DB'],
    "user": os.environ['POSTGRES_USER'],
    "password": os.environ['POSTGRES_PASSWORD']
}
hp = Hydroplane("hydroPLANE", 8053, hydroplane_postgres_env)


# HYDROPLANE GUI ROUTER
@hydro.callback(
    Output('page-content', 'children'),
    Input('url', 'pathname')
)
def display_page(pathname):  
    return hydro_router(hp, pathname)




from hydroplane.hydro import hydro
from hydroplane.components.card import HydroCard
from dash import html, dcc
from dash.dependencies import Input, Output, State, MATCH, ALL 

class HydroOnboarding:
    def __init__(self):
        self.cards = HydroCard()

    def hydro_onboarding_guide(self):
        card_body = html.Section(id="slideshow", children=[
            html.Div(id="slideshow-container", children=[
                html.Div(id="image"),
                dcc.Interval(id='interval', interval=3000)
            ])
        ])
        onboarding = self.cards.hydro_plain_card(1, "large", card_body)
        return onboarding



@hydro.callback(
    Output('image', 'children'),
    Input('interval', 'n_intervals')
)
def onboarding_carousel(n):
    if n == None or n % 3 == 1:
        img = html.Img(src="http://placeimg.com/625/225/any")
    elif n % 3 == 2:
        img = html.Img(src="http://placeimg.com/625/225/animals")
    elif n % 3 == 0:
        img = html.Img(src="http://placeimg.com/625/225/arch")
    else:
        img = "None"
    return img


import dash, os

external_stylesheets = [
    {
        'href': 'https://use.fontawesome.com/releases/v5.8.1/css/all.css',
        'rel': 'stylesheet',
        'integrity': 'sha384-50oBUHEmvpQ+1lW4y57PTFmhCaXp0ML5d60M1M7uH2+nqUivzIebhndOJK28anvf',
        'crossorigin': 'anonymous'
    }
]
 
assets_path = os.path.dirname(os.path.abspath(__file__)) + '/assets'  
hydro = dash.Dash(__name__, external_stylesheets=external_stylesheets) 
hydro.title = "HYDROPLANE"
hydro.config.suppress_callback_exceptions=True



# HYDRO ROUTING 
def hydro_url_route_parser(url): 
    print(url)
    if url == None or url == "/":
        return None
    if url[0] == "/":
        url = url[1:]
    if url[-1] == "/":
        url = url[:-1] 
    return url.split("/")

def hydro_router(hp, url):
    # PARSE CURRENT PAGE URL
    parsed_url = hydro_url_route_parser(url) 
    # HYDRO RESERVED ROUTES - HOME PAGE
    if url == "/" or url == "":
        return "HYDRO HOME PAGE"

    # HYDRO RESERVED ROUTES - QUICK START
    if parsed_url == ["quick-start"]:
        return "QUICK START PAGE"

    # HYDRO RESERVED ROUTES - DATA SOURCE WIZARD
    elif parsed_url == ["wizard", "data-source"]:
        return "WIZARD PAGE"
    elif parsed_url == ["wizard", "data-source", "add"]:
        return "ADD PAGE"
    elif parsed_url[:-1] == ["wizard", "data-source", "edit"]:
        edit_id = parsed_url[-1]
        return "EDIT PAGE" 

    # HYDRO RESERVED ROUTES - QUERY WIZARD
    elif parsed_url == ["wizard", "query"]:
        return "WIZARD PAGE"
    elif parsed_url == ["wizard", "query", "add"]:
        return "ADD PAGE"
    elif parsed_url[:-1] == ["wizard", "query", "edit"]:
        edit_id = parsed_url[-1]
        return "EDIT PAGE"

    # HYDRO RESERVED ROUTES - CARD WIZARD
    elif parsed_url == ["wizard", "card"]:
        return "WIZARD PAGE"
    elif parsed_url == ["wizard", "card", "add"]:
        return "ADD PAGE"
    elif parsed_url[:-1] == ["wizard", "card", "edit"]:
        edit_id = parsed_url[-1]
        return "EDIT PAGE"

    # HYDRO RESERVED ROUTES - PAGE WIZARD
    elif parsed_url == ["wizard", "page"]:
        return "WIZARD PAGE"
    elif parsed_url == ["wizard", "page", "add"]:
        return "ADD PAGE"
    elif parsed_url[:-1] == ["wizard", "page", "edit"]:
        edit_id = parsed_url[-1]
        return "EDIT PAGE"

    # HYDRO RESERVED ROUTES - COLLECTION WIZARD
    elif parsed_url == ["wizard", "collection"]:
        return "WIZARD PAGE"
    elif parsed_url == ["wizard", "collection", "add"]:
        return "ADD PAGE"
    elif parsed_url[:-1] == ["wizard", "collection", "edit"]:
        edit_id = parsed_url[-1]
        return "EDIT PAGE"
    else:
        return "TEST"
    

    # HYDRO CUSTOM PAGE ROUTES
    # routes = hp.LIST_AVAILABLE PAGES / ROUTES
    # for route in routes:
    #     parsed_route = hydro_url_route_parser(route)
    #     if parsed_route == parsed_url:
    #         return "QUERY PAGE URL AND GET PAGE ID, AND BUILD PAGE"

    


from hydroplane.hydro import hydro
from dash import html, dcc, callback_context
from dash.dependencies import Input, Output, State, MATCH, ALL 


class HydroNavigation:
    def __init__(self, hydro_name, messages_enabled, alarms_enabled): 
        self.hydro_name = hydro_name
        self.messages_enabled = messages_enabled
        self.alarms_enabled = alarms_enabled


    # TOP NAVIGATION
    def hydro_top_navigation_profile(self):
        # Profile
        return html.Li(className="nav-item dropdown no-caret dropdown-user me-3 me-lg-4", children=[
            html.Button(id={
                                'type': 'top-nav-dropdown-btn',
                                'index': 3
                            }, className="btn btn-icon btn-transparent-dark dropdown-toggle avatar", n_clicks=0, children=[
                html.Img(className="avatar-img img-fluid", src=hydro.get_asset_url('img/illustrations/profiles/profile-1.png'))
            ]),
            html.Div(id={
                            'type': 'top-nav-dropdown-output',
                            'index': 3
                        }, style={"display": "none"}, className="dropdown-menu dropdown-menu-end border-0 shadow", children=[
                html.H6(className="dropdown-header d-flex align-items-center", children=[
                    html.Img(className="dropdown-user-img", src=hydro.get_asset_url('img/illustrations/profiles/profile-1.png')),
                    html.Div(className="dropdown-user-details", children=[
                        html.Div(className="dropdown-user-details-name", children=[
                            "John Doe"
                        ]),
                        html.Div(className="dropdown-user-details-email", children=[
                            "jdoe@email.com"
                        ])
                    ])
                ]),
                html.Div(className="dropdown-divider"),
                html.Button(id="account-settings", n_clicks=0, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-sliders-h")
                    ]),
                    "Settings"
                ]),
                html.Div(className="dropdown-divider"),
                html.Div(style={"display": "inline-flex"}, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-hat-wizard")
                    ]),
                    dcc.Link("Data Source Wizard", href="/wizard/data-source/add")
                ]), 
                html.Div(style={"display": "inline-flex"}, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-hat-wizard")
                    ]),
                    dcc.Link("Query Wizard", href="/wizard/query/add")
                ]),  
                html.Button(id="card-wizard", n_clicks=0, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-hat-wizard")
                    ]),
                    dcc.Link("Card Wizard", href="/wizard/card")
                ]),
                html.Button(id="page-wizard", n_clicks=0, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-hat-wizard")
                    ]),
                    dcc.Link("Page Wizard", href="/wizard/page")
                ]),
                html.Button(id="collection-wizard", n_clicks=0, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-hat-wizard")
                    ]),
                    dcc.Link("Collection Wizard", href="/wizard/collection")
                ]),
                html.Div(className="dropdown-divider"),
                html.Button(id="collection-system-health", n_clicks=0, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-heartbeat")
                    ]),
                    "System Health"
                ]),
                html.Button(id="collection-users", n_clicks=0, className="dropdown-item", children=[
                    html.Div(className="dropdown-item-icon", children=[
                        html.I(className="fas fa-heartbeat")
                    ]),
                    "User Management"
                ])
            ])
        ])


    def hydro_top_navigation_messages(self):
        # Message Center
        return html.Li(className="nav-item dropdown no-caret d-none d-sm-block me-3 dropdown-notifications", children=[
            html.Button(id={
                                'type': 'top-nav-dropdown-btn',
                                'index': 2
                            }, n_clicks=0, className="btn btn-icon btn-transparent-dark dropdown-toggle", children=[
                html.I(className="far fa-envelope")
            ]),
            html.Div(id={
                            'type': 'top-nav-dropdown-output',
                            'index': 2
                        }, style={"display": "none"}, className="dropdown-menu dropdown-menu-end border-0 shadow", children=[
                html.H6(className="dropdown-header dropdown-notifications-header", children=[
                    html.I(className="me-2 fas fa-bell"),
                    "Message Center"
                ]),
                html.A(className="dropdown-item dropdown-notifications-item", children=[
                    html.Img(className="dropdown-notifications-item-img", src="assets/img/illustrations/profiles/profile-2.png"),
                    html.Div(className="dropdown-notifications-item-content", children=[
                        html.Div(className="dropdown-notifications-item-content-text", children=[
                            "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
                        ]),
                        html.Div(className="dropdown-notifications-item-content-details", children=[
                            "Thomas Wilcox · 58m"
                        ])
                    ])
                ])
            ])
        ])

    def hydro_top_navigation_alerts(self):
        # Alerts
        return html.Li(className="nav-item dropdown no-caret d-none d-sm-block me-3 dropdown-notifications", children=[
            html.Button(id={
                                'type': 'top-nav-dropdown-btn',
                                'index': 1
                            }, n_clicks=0, className="btn btn-icon btn-transparent-dark dropdown-toggle", children=[
                html.I(className="far fa-bell")
            ]),
            html.Div(id={
                            'type': 'top-nav-dropdown-output',
                            'index': 1
                        }, style={"display": "none"}, className="dropdown-menu dropdown-menu-end border-0 shadow", children=[
                html.H6(className="dropdown-header dropdown-notifications-header", children=[
                    html.I(className="fas fa-bell"),
                    "Alerts Center"
                ]),
                html.A(className="dropdown-item dropdown-notifications-item", children=[
                    html.Div(className="dropdown-notifications-item-icon bg-warning", children=[
                        html.I(className="fas fa-bell")
                    ]),
                    html.Div(className="dropdown-notifications-item-content", children=[
                        html.Div(className="dropdown-notifications-item-content-details", children=[
                            "December 29, 2021"
                        ]),
                        html.Div(className="dropdown-notifications-item-content-text", children=[
                            "This is an alert message. It's nothing serious, but it requires your attention."
                        ])
                    ])
                ])
            ])
        ])

    def hydro_top_navigation(self, toggle=True):
        top_nav_list = []
        if self.alarms_enabled:
            top_nav_list.append(self.hydro_top_navigation_alerts())
        if self.messages_enabled:
            top_nav_list.append(self.hydro_top_navigation_messages())
        top_nav_list.append(html.Div(className="topbar-divider d-none d-sm-block"))
        top_nav_list.append(self.hydro_top_navigation_profile())
        top_nav_toggle = html.Div(children=[
            html.Br(),
            html.Button(id="sidebar-toggle-btn", n_clicks=0, style={"display": "none"}, className="btn btn-icon btn-transparent-dark order-1 order-lg-0 me-2 ms-lg-2 me-lg-0", children=[ 
                        html.I(className="fas fa-bars") 
                    ])
        ])
        if toggle:
            top_nav_toggle = html.Button(id="sidebar-toggle-btn", n_clicks=0, className="btn btn-icon btn-transparent-dark order-1 order-lg-0 me-2 ms-lg-2 me-lg-0", children=[ 
                        html.I(className="fas fa-bars") 
                    ])
        return html.Nav(className="topnav navbar navbar-expand shadow justify-content-between justify-content-sm-start navbar-light bg-white", id="sidenavAccordion", children=[
                    
                    top_nav_toggle,
                    # Branding / Logo
                    html.A(className="navbar-brand pe-3 ps-4 ps-lg-2", children=[
                        self.hydro_name
                    ]),
                    # Top Right Features
                    html.Ul(id="nav-top-right-features", style={"display": "flex"}, className="navbar-nav align-items-center ms-auto", children=top_nav_list)
                ])




# TOP NAVIGATION CALLBACKS
@hydro.callback(
    Output({'type': 'top-nav-dropdown-output', 'index': MATCH}, 'style'),
    Output({'type': 'top-nav-dropdown-btn', 'index': MATCH}, 'children'), 
    Input({'type': 'top-nav-dropdown-btn', 'index': MATCH}, 'n_clicks'), 
    Input({'type': 'top-nav-dropdown-output', 'index': MATCH}, 'style'),
    Input({'type': 'top-nav-dropdown-btn', 'index': MATCH}, 'children'),
)
def display_dropdown_output(btn_clicks, style, children): 
    ctx = callback_context  
    button_clicked = ctx.triggered[0]["prop_id"].split(".")[0]  
    if 'top-nav-dropdown-btn' in button_clicked and style["display"] == "none":
        return {"display": "block"}, children
    return {"display": "none"}, children


DATA_SOURCES_TABLE = """
                    CREATE TABLE IF NOT EXISTS data_sources (
                        data_source_id SERIAL PRIMARY KEY,
                        data_source_name varchar(40) UNIQUE NOT NULL,
                        data_source_endpoint varchar(100) NOT NULL,
                        data_source_port varchar(8) NOT NULL,
                        data_source_type varchar(20) NOT NULL
                    )
                    """

QUERIES_TABLE = """
                CREATE TABLE IF NOT EXISTS queries (
                    query_id SERIAL PRIMARY KEY,
                    query_name varchar(40) UNIQUE NOT NULL, 
                    query_data_source_name varchar(40) NOT NULL,
                    query_object varchar(60) NOT NULL,
                    query_object_dimensions json NOT NULL
                )
                """


CARDS_TABLE = """
                CREATE TABLE IF NOT EXISTS cards (
                    card_id SERIAL PRIMARY KEY,
                    card_name varchar(40) UNIQUE NOT NULL,  
                    card_type varchar(40) NOT NULL,
                    card_query varchar(40) NOT NULL,
                    card_chart_type varchar(40) NOT NULL,
                    card_chart_dimensions json NOT NULL,
                    card_tbl_dimensions json NOT NULL
                )
                """

PAGES_TABLE = """
                CREATE TABLE IF NOT EXISTS pages (
                    page_id SERIAL PRIMARY KEY,
                    page_name varchar(40) UNIQUE NOT NULL,  
                    page_endpoint varchar(40) NOT NULL, 
                    page_card_layout json NOT NULL
                )
                """

COLLECTIONS_TABLE = """
                    CREATE TABLE IF NOT EXISTS collections (
                        collection_id SERIAL PRIMARY KEY,
                        collection_name varchar(40) UNIQUE NOT NULL,  
                        collection_icon varchar(40) NOT NULL, 
                        collection_pages json NOT NULL
                    )
                    """

# FIX TIMESTAMP
LOGS_TABLE = """
                    CREATE TABLE IF NOT EXISTS logs (
                        log_id SERIAL PRIMARY KEY,
                        log_timestamp varchar(40) UNIQUE NOT NULL,  
                        log_type varchar(40) UNIQUE NOT NULL,  
                        log_msg varchar(40) NOT NULL 
                    )
                    """


SETTINGS_TABLE = """
                    CREATE TABLE IF NOT EXISTS settings (
                        settings_id SERIAL PRIMARY KEY,
                        retention_period INTEGER UNIQUE NOT NULL 
                    )
                    """


import pathlib
from setuptools import setup, find_packages

# # The directory containing this file
# HERE = pathlib.Path(__file__).parent

# # The text of the README file
# README = (HERE / "README.md").read_text()

setup(
   name='hydroplane',
   version='0.0.61',
   author='Ryan Moos',
   author_email='ryan@moos.engineering',
   packages=find_packages(),
   scripts=['bin/hydro'],
   url='http://pypi.python.org/pypi/hydroplane/',
   license='LICENSE',
   description='A Data Exploration Custom Dashboarding Framework',
   long_description=open('README.md').read(),
   long_description_content_type='text/markdown',
   include_package_data=True,
   install_requires=[
       "dash", 
       "pandas",
       "numpy",
       "cython",
       "pydruid",
       "requests",
       "psycopg2-binary",
       "dash-editor-components",
       "python-dotenv"
   ],
)


import os, requests
from pydruid.client import *
from pydruid.utils.aggregators import * 
from pydruid.utils.postaggregator import * 


class HydroDruid:
    def __init__(self, druid_broker_endpoint, druid_broker_port):
        self.druid_broker_endpoint = druid_broker_endpoint
        self.druid_broker_port = druid_broker_port
        self.druid = self.druid_client()
        self.druid_status = self.druid_status()

    def druid_client(self):
        client = PyDruid('http://' + self.druid_broker_endpoint + ':' + self.druid_broker_port, 'druid/v2')
        return client

    def druid_status(self): 
        url = "http://" + self.druid_broker_endpoint + ":" + self.druid_broker_port + "/status/health" 
        response = requests.get(url) 
        if response.status_code == 200 and response.json():
            self.druid_status = True
            return True
        self.druid_status = False
        return False

    def druid_data_sources(self): 
        url = "http://" + self.druid_broker_endpoint + ":" + self.druid_broker_port + "/druid/v2/datasources" 
        response = requests.get(url) 
        return response.json()
    # def druid_timeseries_query():

    def druid_data_source_dimensions(self, druid_datasource_name): 
        url = "http://" + self.druid_broker_endpoint + ":" + self.druid_broker_port + "/druid/v2/datasources/{}".format(druid_datasource_name) 
        response = requests.get(url) 
        return response.json()

    def druid_time_series_query(self, datasource, granularity=None, intervals=None, aggregations=None, post_aggregations=None, filter=None):  
        ts = self.druid.timeseries(
            datasource=datasource,
            granularity=granularity,
            intervals=intervals,
            aggregations=aggregations,
            post_aggregations=post_aggregations,
            filter=filter
        )
        df = self.druid.export_pandas()

        # CLEAN UP DF
        # if granularity == "day":
        #     df['timestamp'] = df['timestamp'].map(lambda x: x.split('T')[0])
        # else:
        #     print("DO SOMETHING WITH TIME AND TZ")
        return df



    def druid_group_by_query(self, datasource, granularity=None, intervals=None, aggregations=None, post_aggregations=None, filter=None, limit_spec=None):  
        ts = self.druid.groupby(
            datasource=datasource,
            granularity=granularity,
            intervals=intervals,
            aggregations=aggregations,
            post_aggregations=post_aggregations,
            filter=filter,
            limit_spec = limit_spec
        )
        df = self.druid.export_pandas()
        # CLEAN UP DF
        # if granularity == "day":
        #     df['timestamp'] = df['timestamp'].map(lambda x: x.split('T')[0])
        # else:
        #     print("DO SOMETHING WITH TIME AND TZ")
        return df


from hydroplane.hydro import hydro
from dash import html, dcc, callback_context
from dash.dependencies import Input, Output, State, MATCH, ALL 

class HydroCard:
    def __init__(self) -> None:
        pass

    def hydro_plain_card(self, card_idx, card_size, body): 
        className = "col-lg-12 mb-4"
        if card_size == "large":
            className = "col-lg-12 mb-4"
        elif card_size == "medium":
            className = "col-lg-8 mb-4"
        elif card_size == "small":
            className = "col-lg-6 mb-4" 
        elif card_size == "preview-1":
            className = "col-lg-8 mb-4" 
        elif card_size == "preview-2":
            className = "col-lg-4 mb-4" 
        else:
            className = "col-lg-12 mb-4"
        return html.Div(className=className, children=[ 
                html.Div(id="card-id", style={"display":"none"}, children=card_idx),
                html.Div(className="card", children=[ 
                    html.Div(id={'type':'card-plain-body','index':card_idx}, className="card-body", style={"display":"block"}, children=body)
                ])
            ])
    def hydro_simple_card(card_idx, card_size, header, body): 
        className = "col-lg-12 mb-4"
        if card_size == "large":
            className = "col-lg-12 mb-4"
        elif card_size == "medium":
            className = "col-lg-8 mb-4"
        elif card_size == "small":
            className = "col-lg-6 mb-4" 
        return html.Div(className=className, children=[ 
                html.Div(id="card-id", style={"display":"none"}, children=card_idx),
                html.Div(className="card", children=[
                    html.Div(className="card-header", children=[ 
                        header 
                    ]),
                    
                    html.Div(id={'type':'card-simple','index':card_idx}, className="card-body", style={"display":"block"}, children=body) 
                ])
            ])


from hydroplane.hydro import hydro, hydro_router
from hydroplane.components.navigation import HydroNavigation
from hydroplane.components.card import HydroCard
from hydroplane.components.onboard import HydroOnboarding
from hydroplane.components.simple import HydroSimple
from hydroplane.data_sources.postgres import HydroPostgres, DATA_SOURCES_TABLE, QUERIES_TABLE, CARDS_TABLE, PAGES_TABLE, COLLECTIONS_TABLE, LOGS_TABLE, SETTINGS_TABLE
from hydroplane.data_sources.druid import HydroDruid

import dash, psycopg2, sys, json, hashlib, uuid, base64, os
from dash import html, dcc
from dash.dependencies import Input, Output, State, MATCH, ALL 
from urllib.parse import urlparse


random_value = uuid.uuid4().hex

hydroplane_postgres_environment = {
    "host": os.getenv('POSTGRES_HOST', 'pg_host'),
    "port": os.getenv('POSTGRES_PORT', 'pg_port'),
    "database": os.getenv('POSTGRES_DB', 'pg_db'),
    "user": os.getenv('POSTGRES_USER', 'pg_user'),
    "password": os.getenv('POSTGRES_PASSWORD', 'pg_pw')
}

class Hydroplane:
    def __init__(self, hydro_name, hydro_port, hydro_debug=True, hydro_threaded=True, hydro_archive=None, hydro_messages=None, hydro_alerts=None):
        # HYDRO INPUTTS
        self.hydro_name = hydro_name
        self.hydro_port = hydro_port 
        self.hydro_debug = hydro_debug
        self.hydro_threaded = hydro_threaded
        self.hydro_archive = hydro_archive
        self.hydro_messages = hydro_messages
        self.hydro_alerts = hydro_alerts

        # HYDRO COMPONENTS
        self.hydro_components_navigation = HydroNavigation(self.hydro_name, self.hydro_messages, self.hydro_alerts)
        self.hydro_components_card = HydroCard()
        self.hydro_components_chart = None
        self.hydro_components_table = None
        self.hydro_components_onboarding = HydroOnboarding()
        self. hydro_components_simple = HydroSimple()
        # HYDRO WIZARDS
        self.hydro_wizard_data_source = None
        self.hydro_wizard_query = None
        self.hydro_wizard_card = None
        self.hydro_wizard_page = None
        self.hydro_wizard_collection = None
        # HYDRO POSTGRES
        self.hydro_data_source_postgres = HydroPostgres(hydroplane_postgres_environment["host"], hydroplane_postgres_environment["port"], hydroplane_postgres_environment["database"], hydroplane_postgres_environment["user"], hydroplane_postgres_environment["password"])
 
    def hydro_wireframe(self, hydro_top_navigation): 
        wireframe = html.Div(id="wrapper", children=[
                        html.Div(id="wrapper-children", className="nav-fixed sidenav-toggled", children = [ 
                            # URL
                            dcc.Location(id='url', refresh=False), 
                            # TOP NAVIGATION  
                            hydro_top_navigation,

                            # MAIN CONTENT (PAGE + SIDE NAV)
                            html.Div(id="layoutSidenav", children=[
                                # SIDE NAVIGATION
                                html.Div(id="hydro-sidenav-navigation"),
                                # PAGE CONTENT 
                                html.Div(id="layoutSidenav_content", children=[
                                    html.Div(id="hydro-page-content", className="container-xl px-4 mt-5")
                                ])
                            ])
                        ]) 
                    ])
        return wireframe
        
    def simple_hydro(self):
        hydro_top_navigation = self.hydro_components_navigation.hydro_top_navigation(toggle=False)
        hydro.layout = self.hydro_wireframe(hydro_top_navigation)
        hydro.run_server(host="0.0.0.0", port="8059", debug=self.hydro_debug, threaded=self.hydro_threaded)

    def start_up(self):
        # check if postgres is available, if not then wait and retry 
        # CREATE POSTGRES CONNECTION
        try:
            conn = self.hydro_data_source_postgres.postgres_create_connection() 
        except psycopg2.OperationalError:
            # print("Oops!", sys.exc_info()[0], "occurred.")
            print("ERROR: POSTGRES CONNECTION FAILURE - psycopg2.OperationalError")
        else:
            # CREATE IF DOES NOT EXIST POSTGRES TABLES
            self.hydro_data_source_postgres.postgres_create_table(conn, "DATA SOURCES", DATA_SOURCES_TABLE)
            self.hydro_data_source_postgres.postgres_create_table(conn, "QUERIES", QUERIES_TABLE)
            self.hydro_data_source_postgres.postgres_create_table(conn, "CARDS", CARDS_TABLE)
            self.hydro_data_source_postgres.postgres_create_table(conn, "PAGES", PAGES_TABLE)
            self.hydro_data_source_postgres.postgres_create_table(conn, "COLLECTIONS", COLLECTIONS_TABLE)
            self.hydro_data_source_postgres.postgres_create_table(conn, "LOGS", LOGS_TABLE)
            self.hydro_data_source_postgres.postgres_create_table(conn, "SETTINGS", SETTINGS_TABLE)  

            # LOOK FOR EXISTING COLLECTIONS, PAGES, CARDS, QUERIES, DATA SOURCES

            # IF NONE ONBOARD USER
            wireframe_body_onboarding = self.hydro_components_onboarding.hydro_onboarding_guide()

            # CLOSE POSTGRES CONNECTION
            self.hydro_data_source_postgres.postgres_close_connection(conn) 


            # SET HYDRO LAYOUT
            hydro_top_navigation = self.hydro_components_navigation.hydro_top_navigation()
            hydro_body = wireframe_body_onboarding
            hydro.layout = self.hydro_wireframe(hydro_top_navigation)
            # RUN WEBSERVER
            hydro.run_server(host="0.0.0.0", port="8053", debug=self.hydro_debug, threaded=self.hydro_threaded)
            # hydro.run_server(host="0.0.0.0", port=str(self.hydro_port), debug=self.hydro_debug, threaded=self.hydro_threaded)
        





# HYDROPLANE GUI ROUTER
@hydro.callback(
    Output('hydro-page-content', 'children'),
    Output('hydro-sidenav-navigation', 'children'),
    Input('url', 'pathname')
)
def display_page(pathname):   
    
    # LOOK FOR EXISTING COLLECTIONS, PAGES, CARDS, QUERIES, DATA SOURCES 
    # IF NONE ONBOARD USER
    wireframe_body_onboarding = HydroOnboarding().hydro_onboarding_guide()
    if pathname == "/onboarding": 
        print(wireframe_body_onboarding) 
        return wireframe_body_onboarding, ""
    x = hydro_router(1, pathname)
    print(x)
    return x, ""


@hydro.callback(
    Output('wrapper-children', 'className'),
    Output('sidebar-toggle-btn', 'n_clicks'),
    Input('sidebar-toggle-btn', 'n_clicks'), 
    Input('wrapper-children', 'className')
)
def sidebar_toggle(btn, current_cls):
    if btn > 0 and current_cls == "nav-fixed sidenav-toggled":
        return "nav-fixed", 0 
    else:
        return "nav-fixed sidenav-toggled", 0


import os, psycopg2
import pandas.io.sql as psql
from hydroplane.data_sources.postgres.postgres_tables import DATA_SOURCES_TABLE, QUERIES_TABLE, CARDS_TABLE, PAGES_TABLE, COLLECTIONS_TABLE, LOGS_TABLE, SETTINGS_TABLE



class HydroPostgres:
    def __init__(self, host, port, database, user, password):
        self.host = host
        self.port = port
        self.database = database
        self.user = user
        self.password = password

    def postgres_create_connection(self):
        conn = psycopg2.connect(host=self.host, port=self.port, database=self.database, user=self.user, password=self.password)
        return conn


    def postgres_close_connection(self, conn):
        conn.close()   


    def postgres_create_table(self, conn, tbl_name, tbl_sql):  
        cur = conn.cursor()
        cur.execute(tbl_sql)  
        conn.commit() 
        log_type = "INFO"
        log_msg = "TABLE: " + tbl_name + " - Acitve"
        # self.postgres_add_log(conn, log_type, log_msg)


    def postgres_add_log(self, conn, log_type, log_msg):
        cur = conn.cursor()
        # CREATE LOGS TABLE IF IT DOES NOT EXIST 
        self.postgres_create_table(conn, "LOGS", LOGS_TABLE)
        # ADD LOG 
        log_timestamp = ""
        log_sql = '''INSERT INTO logs (log_timestamp, log_type, log_msg) VALUES ('{log_timestamp}', '{log_type}', '{log_msg}')'''.format(log_timestamp=log_timestamp, log_type=log_type, log_msg=log_msg) 
        cur.execute(log_sql)  
        conn.commit() 

